/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Patrick Boenzli
*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_IMPORTER

#include "md3_model.h"

#include "md3_data.h"

#include "md3_animation_cfg.h"

namespace md3
{

  /**
   * md3 model
   */
  MD3Model::MD3Model(std::string filename, float scaling)
  {
    this->md3Data = new MD3Data(filename, filename, scaling);

    MD3AnimationCfg cfg("/home/boenzlip/tmp/q3/Downloads/MOH/q3mdl-alien3/models/players/alien3/animation.cfg");
  }



  MD3Model::~MD3Model()
  {}



  /**
   * this draws the md3 model
   */
  void MD3Model::draw()
  {
    //draw current bone frame
#if 0
    if( this->bDrawBones)
    {
      //get bone frame, interpolate if necessary
      if( model.interpolationFraction!=0.0 && model.currentFrame!=model.nextFrame)
        //interpolate bone frame
        drawBoneFrame(interpolateBoneFrame(model.boneFrames[model.currentFrame], model.boneFrames[model.nextFrame], model.interpolationFraction));
      else
        //stick with current bone frame
        drawBoneFrame(model.boneFrames[model.currentFrame]);
    }

    //draw all meshes of current frame of this model
    for (int i=0; i<model.meshNum; i++) {
      MD3GLMesh mesh = (MD3GLMesh)model.meshes[i];

      gl.glBlendFunc(mesh.GLSrcBlendFunc, mesh.GLDstBlendFunc);
      gl.glDepthMask(mesh.GLDepthMask);
      if (mesh.textureNum > 0 && mesh.textures[0]!=null)
        gl.glBindTexture(GLEnum.GL_TEXTURE_2D, ((MD3GLTexture)mesh.textures[0]).bind);
      else
        gl.glBindTexture(GLEnum.GL_TEXTURE_2D, 0);

      //get mesh frame, do interpolation if necessary
      Vec3[] frame;
      if (model.interpolationFraction!=0.0 && model.currentFrame!=model.nextFrame)
        //interpolate mesh frame between the 2 current mesh frames
        frame=interpolateMeshFrame(mesh.meshFrames[model.currentFrame], mesh.meshFrames[model.nextFrame], model.interpolationFraction);
      else
        //no interpolation needed, just draw current frame
        frame=mesh.meshFrames[model.currentFrame];

      drawMesh(mesh, frame);

      //draw vertex normals
      if (canvas.showVertexNormals) {
                                //get vertex normals, interpolate if necessary
        if (model.interpolationFraction!=0.0 && model.currentFrame!=model.nextFrame)
                //interpolate vertex normals
          drawVertexNormals(frame, interpolateVertexNormals(mesh.meshVertexNormals[model.currentFrame], mesh.meshVertexNormals[model.nextFrame], model.interpolationFraction));
        else
                //stick with current vertex normals
          drawVertexNormals(frame, mesh.meshVertexNormals[model.currentFrame]);
      }
    }

    //draw all models linked to this model

    Iterator it=model.linkedModels();
    while (it.hasNext()) {
      MD3Model child=(MD3Model)it.next();

      //build transformation array m from matrix, interpolate if necessary
      float[] m=new float[16];
      MD3Tag currFrameTag=model.boneFrames[model.currentFrame].tags[child.getParentTagIndex()];
      if (model.interpolationFraction!=0.0f && model.currentFrame!=model.nextFrame) {
        //we need to interpolate
        MD3Tag nextFrameTag=model.boneFrames[model.nextFrame].tags[child.getParentTagIndex()];
        m=interpolateTransformation(currFrameTag, nextFrameTag, model.interpolationFraction);
      }
      else {
        //no interpolation needed, stay with last transformation
              //OpenGL matrix is in column-major order
        m[0] = currFrameTag.matrix[0][0]; m[4] = currFrameTag.matrix[0][1]; m[8] = currFrameTag.matrix[0][2]; m[12] = currFrameTag.position.x;
        m[1] = currFrameTag.matrix[1][0]; m[5] = currFrameTag.matrix[1][1]; m[9] = currFrameTag.matrix[1][2]; m[13] = currFrameTag.position.y;
        m[2] = currFrameTag.matrix[2][0]; m[6] = currFrameTag.matrix[2][1]; m[10]= currFrameTag.matrix[2][2]; m[14] = currFrameTag.position.z;
        m[3] = 0.0f;                                              m[7] = 0.0f;                                              m[11]= 0.0f;                            m[15] = 1.0f;
      }

      //switch to child coord system and draw child
      gl.glPushMatrix();
      gl.glMultMatrixf(m);
      child.accept(this);
      gl.glPopMatrix();
    }
#endif
  }




}
